@@ -212,7 +212,9 @@
   (declare (type function fun)
            (type fixnum start end))
   (declare (dynamic-extent fun))
-  (let* ((start (* start 2)) (end (* end 2)) ; Utter garbage conversion
+  ;; START/END are passed as fixnum-encoded raw words to ensure no boxing
+  (let* ((start (get-lisp-obj-address start))
+         (end (get-lisp-obj-address end))
          (first-byte (floor (- start dynamic-space-start)
                            (ash 8 n-lowtag-bits)))
         (last-byte (ceiling (- end dynamic-space-start n-lowtag-bits)
@@ -234,8 +236,14 @@
                        (aver (not (logtest (the fixnum size) lowtag-mask)))
                        ;; TODO: Each line has exactly one generation; should
                        ;; check that in the outer loop instead.
-                       (when (logbitp (generation-of obj) generation-mask)
-                         (funcall fun obj typecode size))))))))))
+                       ;; This code SHOULD work but does not:
+                       ;;   (let ((gen (the (not null) (generation-of obj))))
+                       ;;    (when (logbitp gen generation-mask)
+                       ;; So it was using the 'default' arg to gc_gen_of.
+                       ;; But why??? We're in a generational space aren't we?
+                       (let ((gen (generation-of obj)))
+                         (when (and gen (logbitp gen generation-mask))
+                           (funcall fun obj typecode size)))))))))))
 
 ;;; Access to the GENCGC page table for better precision in
 ;;; MAP-ALLOCATED-OBJECTS
@@ -658,16 +666,20 @@ We could try a few things to mitigate this:
        (when (or (eql type instance-widetag)
                  (eql type funcallable-instance-widetag))
          (incf total-objects)
-         (let* ((layout (if (eql type funcallable-instance-widetag)
-                            (%fun-layout obj)
-                            (%instance-layout obj)))
-                (classoid (layout-classoid layout))
-                (found (ensure-gethash classoid totals (cons 0 0)))
-                (size size))
-           (declare (fixnum size))
-           (incf total-bytes size)
-           (incf (the fixnum (car found)))
-           (incf (the fixnum (cdr found)) size))))
+         (block nil
+           (let* ((layout (if (eql type funcallable-instance-widetag)
+                              (%fun-layout obj)
+                              (%instance-layout obj)))
+                  (classoid (if (zerop (get-lisp-obj-address layout))
+                                ;; Don't crash on partially allocated instances
+                                (return)
+                                (layout-classoid layout)))
+                  (found (ensure-gethash classoid totals (cons 0 0)))
+                  (size size))
+             (declare (fixnum size))
+             (incf total-bytes size)
+             (incf (the fixnum (car found)))
+             (incf (the fixnum (cdr found)) size)))))
      space)
     (let* ((sorted (sort (%hash-table-alist totals) #'> :key #'cddr))
            (interesting (if top-n
@@ -821,11 +833,11 @@ We could try a few things to mitigate this:
   ;; something and then see it again after GC transports it higher. The only way to
   ;; allow consing in the predicate would be to use dedicated "arenas" for new
   ;; allocations, that being a concept which we do not now - and may never - support.
-  (sb-int:dx-flet ((wantp (obj widetag size)
-                     (and (or (not type) (eql widetag type))
-                          (or (not smaller) (<= size smaller))
-                          (or (not larger) (>= size larger))
-                          (or (not test) (funcall test obj)))))
+  (flet ((wantp (obj widetag size)
+           (and (or (not type) (eql widetag type))
+                (or (not smaller) (<= size smaller))
+                (or (not larger) (>= size larger))
+                (or (not test) (funcall test obj)))))
     ;; Unless COUNT is smallish, always start by counting. Don't just trust the user
     ;; because s/he might specify :COUNT huge-num which is acceptable provided that
     ;; huge-num is an INDEX which could either exhaust the heap, or at least be
@@ -878,7 +890,7 @@ We could try a few things to mitigate this:
                     ;; Explicitly skip non-pointer words. The callable that
                     ;; SB-INTROSPECT provides ignores immediate values anyway.
                     (when (and (is-lisp-pointer word)
-                               (not (zerop (sb-di::valid-lisp-pointer-p (int-sap word)))))
+                               (not (zerop (sb-di::valid-tagged-pointer-p (int-sap word)))))
                       (let ((obj (%make-lisp-obj word)))
                         (unless (memq obj seen)
                           (push obj seen)
@@ -1001,7 +1013,7 @@ We could try a few things to mitigate this:
                ;; comment in conservative_root_p() in gencgc as to why that alone
                ;; would be inadequate- we require a properly tagged descriptor
                ;; to enliven any object other than code.
-               #+immobile-code
+               #+(and immobile-code x86-64)
                `(%make-lisp-obj
                  (alien-funcall (extern-alien "decode_fdefn_rawfun" (function unsigned unsigned))
                                 (logandc2 (get-lisp-obj-address ,obj) lowtag-mask))))
@@ -1267,7 +1279,8 @@ We could try a few things to mitigate this:
          ;; gen0 conses on MIXED pages, but even that is not enough- pinned conses
          ;; will promote but keep their MIXED page type. So don't bother with this.
          #+use-cons-region
-         (let* ((type (slot (deref page-table (find-page-index obj-addr)) 'flags))
+         (let* ((type (logandc2 (slot (deref page-table (find-page-index obj-addr)) 'flags)
+                                32)) ; OPEN_REGION_PAGE_FLAG (due to finalizer thread perhaps)
                 (ok (if (consp obj)
                         (or (= type #b101) ; PAGE_TYPE_CONS
                             (and (eq (car obj) 0) (eq (cdr obj) 0)))
