@@ -1439,3 +1439,42 @@ is replaced with replacement."
                                   (cleanup-mess-up
                                    (block-start-cleanup ep)))))))))
     (write-line "}" stream)))
+
+(defun print-constraint (constraint &optional kind)
+  (let ((kind* (constraint-kind constraint)))
+    (when (or (not kind)
+              (eq kind* kind))
+      (flet ((f (x)
+               (cond ((lambda-var-p x)
+                      (leaf-source-name x))
+                     ((ctype-p x)
+                      (type-specifier x))
+                     (t x))))
+        (format t "~a~a ~a ~a~%" kind*
+                (if (constraint-not-p constraint)
+                    " NOT"
+                    "")
+                (f (constraint-x constraint))
+                (f (constraint-y constraint)))))))
+
+(defun print-constraints (component &optional kind)
+  (do-blocks (block component)
+    (handler-case (progn
+                    (terpri)
+                    (terpri)
+                    (do-conset-elements (con (block-in block))
+                      (print-constraint con kind))
+                    (print-nodes block)
+                    (let ((last (block-last block)))
+                      (cond ((if-p last)
+                             (format t "CONSEQ~%")
+                             (do-conset-elements (con (if-consequent-constraints last))
+                               (print-constraint con kind))
+                             (format t "ALT~%")
+                             (do-conset-elements (con (if-alternative-constraints last))
+                               (print-constraint con kind)))
+                            (t
+                             (do-conset-elements (con (block-out block))
+                               (print-constraint con kind))))))
+      (error (condition)
+        (format t "~&~A...~%" condition)))))
