@@ -83,7 +83,7 @@
                 packagep functionp compiled-function-p not)
   (t) boolean (movable foldable flushable))
 
-(defknown (eq eql %eql/integer) (t t) boolean
+(defknown (eq eql) (t t) boolean
   (movable foldable flushable commutative))
 (defknown (equal equalp) (t t) boolean (foldable flushable recursive))
 
@@ -98,10 +98,9 @@
 (defknown find-classoid (name-for-class &optional t)
   (or classoid null) ())
 (defknown classoid-of (t) classoid (flushable))
-(defknown wrapper-of (t) wrapper (flushable))
-(defknown wrapper-depthoid (wrapper) layout-depthoid (flushable))
-#+64-bit (defknown layout-depthoid (sb-vm:layout) layout-depthoid (flushable always-translatable))
-#+(or x86 x86-64) (defknown (layout-depthoid-ge) (sb-vm:layout integer) boolean (flushable))
+(defknown layout-of (t) layout (flushable))
+#+64-bit (defknown layout-depthoid (layout) layout-depthoid (flushable always-translatable))
+#+(or x86 x86-64) (defknown (layout-depthoid-ge) (layout integer) boolean (flushable))
 (defknown %structure-is-a (instance t) boolean (foldable flushable))
 (defknown structure-typep (t t) boolean (foldable flushable))
 (defknown classoid-cell-typep (t t) boolean (foldable flushable no-verify-arg-count))
@@ -433,7 +432,7 @@
 (defknown (float-digits float-precision) (float) float-digits
   (movable foldable unsafely-flushable))
 (defknown integer-decode-float (float)
-    (values integer float-int-exponent (member -1 1))
+    (values double-float-significand float-int-exponent (member -1 1))
     (movable foldable unsafely-flushable))
 
 (defknown single-float-sign (single-float) single-float (movable foldable flushable))
@@ -458,11 +457,12 @@
 (defknown lognot (integer) integer (movable foldable flushable))
 (defknown logtest (integer integer) boolean (movable foldable flushable commutative))
 (defknown logbitp (unsigned-byte integer) boolean (movable foldable flushable))
-(defknown ash (integer integer) integer
+(defknown (ash ash-inverted) (integer integer) integer
   (movable foldable flushable))
 (defknown %ash/right ((or word sb-vm:signed-word) (mod #.sb-vm:n-word-bits))
   (or word sb-vm:signed-word)
   (movable foldable flushable always-translatable))
+
 (defknown (logcount integer-length) (integer) bit-index
   (movable foldable flushable))
 ;;; FIXME: According to the ANSI spec, it's legal to use any
@@ -560,6 +560,9 @@
 (defknown subseq (proper-sequence index &optional sequence-end) consed-sequence
   (flushable))
 
+(defknown vector-subseq* (vector index sequence-end) (simple-array * (*))
+  (flushable))
+
 (defknown copy-seq (proper-sequence) consed-sequence (flushable)
   :derive-type (sequence-result-nth-arg 0 :preserve-dimensions t))
 
@@ -594,11 +597,14 @@
     vector
   (flushable no-verify-arg-count))
 
+(defknown (possibly-base-stringize possibly-base-stringize-to-heap) (string) simple-string
+  (flushable no-verify-arg-count))
+
 (defknown map (type-specifier (function-designator ((nth-arg 2 :sequence t)
                                                     (rest-args :sequence t))
                                                    (nth-arg 0 :sequence-type t))
                               proper-sequence &rest proper-sequence)
-  consed-sequence (call))
+    consed-sequence (call))
 (defknown %map (type-specifier function-designator &rest sequence) consed-sequence
   (call no-verify-arg-count))
 (defknown %map-for-effect-arity-1 (function-designator sequence) null
@@ -633,8 +639,8 @@
 (defknown (every notany notevery) (function-designator proper-sequence &rest proper-sequence) boolean
   (foldable unsafely-flushable call))
 
-(defknown reduce ((function-designator ((nth-arg 1 :sequence t :key :key :value :initial-value)
-                                        (nth-arg 1 :sequence t :key :key)))
+(defknown reduce ((function-designator ((nth-arg 1 :sequence t :key :key :value (:initial-value :from-end nil))
+                                        (nth-arg 1 :sequence t :key :key :value (:initial-value :from-end t))))
                   proper-sequence &rest t &key (:from-end t)
                   (:start (inhibit-flushing index 0))
                   (:end (inhibit-flushing sequence-end nil))
@@ -1311,7 +1317,7 @@
    (:start (inhibit-flushing index 0))
    (:end (inhibit-flushing sequence-end nil)))
   simple-string (flushable)
-  :derive-type (sequence-result-nth-arg 0 :preserve-dimensions t))
+  :derive-type (sequence-result-nth-arg 0 :preserve-dimensions t :string-designator t))
 
 (defknown (nstring-upcase nstring-downcase nstring-capitalize)
   ((modifying string) &key (:start index) (:end sequence-end))
@@ -1723,6 +1729,7 @@
 
 (defknown (signal warn) (condition-designator-head &rest t) null)
 (defknown error (condition-designator-head &rest t) nil)
+(defknown %simple-error (condition-designator-head t &rest t) nil)
 (defknown cerror (format-control condition-designator-head &rest t) null)
 (defknown invalid-method-error (t format-control &rest t) *) ; FIXME: first arg is METHOD
 (defknown method-combination-error (format-control &rest t) *)
@@ -1877,7 +1884,7 @@
 ;;; special optimizer anyway.
 (defknown %typep (t (or type-specifier ctype)) boolean
   (movable flushable no-verify-arg-count))
-(defknown %instance-typep (t (or type-specifier ctype wrapper)) boolean
+(defknown %instance-typep (t (or type-specifier ctype layout)) boolean
   (movable flushable always-translatable))
 ;;; We should never emit a call to %typep-wrapper
 (defknown %typep-wrapper (t t (or type-specifier ctype)) t
@@ -1891,15 +1898,12 @@
   :derive-type #'result-type-first-arg)
 
 (defknown %cleanup-point (&rest t) t (reoptimize-when-unlinking))
+(defknown %dynamic-extent-start () t)
 (defknown %special-bind (t t) t)
 (defknown %special-unbind (&rest symbol) t)
 (defknown %listify-rest-args (t index) list (flushable))
 (defknown %more-arg-context (t t) (values t index) (flushable))
 (defknown %more-arg (t index) t (flushable))
-(defknown %more-keyword-pair (t fixnum) (values t t) (flushable))
-#+stack-grows-downward-not-upward
-;;; FIXME: The second argument here should really be NEGATIVE-INDEX, but doing that
-;;; breaks the build, and I cannot seem to figure out why. --NS 2006-06-29
 (defknown %more-kw-arg (t fixnum) (values t t))
 (defknown %more-arg-values (t index index) * (flushable))
 
@@ -1915,10 +1919,9 @@
 (defknown %continue-unwind () nil)
 (defknown %throw (t &rest t) nil) ; This is MV-called.
 (defknown %nlx-entry (t) *)
-(defknown %%primitive (t t &rest t) *)
+(defknown %%primitive (t &rest t) *)
 (defknown %pop-values (t) t)
 (defknown %nip-values (t t &rest t) (values))
-(defknown %dummy-dx-alloc (t t) t)
 (defknown %type-check-error (t t t) nil)
 (defknown %type-check-error/c (t t t) nil)
 
@@ -1953,6 +1956,12 @@
 (defknown hairy-data-vector-set (array index t) t (no-verify-arg-count))
 (defknown hairy-data-vector-ref/check-bounds (array index) t (foldable no-verify-arg-count))
 (defknown hairy-data-vector-set/check-bounds (array index t) t (no-verify-arg-count))
+
+(defknown vector-hairy-data-vector-ref (vector index) t (foldable flushable no-verify-arg-count))
+(defknown vector-hairy-data-vector-set (vector index t) t (no-verify-arg-count))
+(defknown vector-hairy-data-vector-ref/check-bounds (vector index) t (foldable no-verify-arg-count))
+(defknown vector-hairy-data-vector-set/check-bounds (vector index t) t (no-verify-arg-count))
+
 (defknown %caller-frame () t (flushable))
 (defknown %caller-pc () system-area-pointer (flushable))
 (defknown %with-array-data (array index (or index null))
@@ -2108,7 +2117,7 @@
 (defknown %program-error (&optional t &rest t) nil ())
 (defknown compiler-error (t &rest t) nil ())
 (defknown (compiler-warn compiler-style-warn) (t &rest t) (values) ())
-(defknown (compiler-mumble note-lossage note-unwinnage) (string &rest t) (values) ())
+(defknown (compiler-mumble note-lossage note-unwinnage) (format-control &rest t) (values) ())
 (defknown (compiler-notify maybe-compiler-notify) ((or format-control symbol) &rest t)
   (values)
   ())
@@ -2240,27 +2249,13 @@
   (defknown round-single (single-float #1#) single-float
       (foldable flushable movable always-translatable)))
 
-(defknown fixnum*
-  (fixnum fixnum t)
-  fixnum
-  (movable always-translatable))
-
-(defknown (signed* signed+ signed-)
-  (sb-vm:signed-word sb-vm:signed-word t)
-  sb-vm:signed-word
-  (movable always-translatable))
-
-(defknown (unsigned* unsigned+ unsigned-)
-  (word word t)
-  word
-  (movable always-translatable))
-
-(defknown (unsigned+signed unsigned-signed)
-  (word sb-vm:signed-word t)
+(defknown (overflow* overflow+ overflow-
+           overflow-ash)
+  (integer integer t)
   integer
   (movable always-translatable))
 
-(defknown (signed-unsigned)
-  (sb-vm:signed-word word t)
+(defknown overflow-negate
+  (integer t)
   integer
   (movable always-translatable))
