@@ -179,7 +179,7 @@
 
 (defun make-number-stack-pointer-tn ()
   #+c-stack-is-control-stack
-  (make-restricted-tn *fixnum-primitive-type* any-reg-sc-number)
+  (sb-c::make-unused-tn)
   #-c-stack-is-control-stack
   (make-normal-tn *fixnum-primitive-type*))
 
@@ -208,7 +208,9 @@
   "Cause a continuable error.  ERROR-CODE is the error to cause."
   (emit-error-break vop cerror-trap (error-number-or-lose error-code) values))
 
-#+sb-safepoint
+;; C header generation can't execute DEFINE-VOP
+;; Perhaps this belongs somewhere different
+#+(and sb-safepoint (not :c-headers-only))
 (define-vop (insert-safepoint)
   (:policy :fast-safe)
   (:translate sb-kernel::gc-safepoint)
@@ -235,39 +237,75 @@
 ;;; then NIL is ok as the input. Indicate this by specifying PERMIT-NIL.
 ;;; With rare exception it should always be permitted, though not on ppc64
 ;;; where it would never be. The safe default is NIL.
-(defun other-pointer-tn-ref-p (tn-ref &optional permit-nil)
+(defun other-pointer-tn-ref-p (tn-ref &optional permit-nil
+                                                immediates-tested)
   (and (sc-is (tn-ref-tn tn-ref) descriptor-reg)
        (not (types-equal-or-intersect
              (tn-ref-type tn-ref)
              (if permit-nil
-                 (specifier-type '(or fixnum
-                                   #+64-bit single-float
-                                   function
-                                   cons
-                                   instance
-                                   character))
-                 (specifier-type '(or fixnum
-                                   #+64-bit single-float
-                                   function
-                                   list
-                                   instance
-                                   character)))))))
+                 (specifier-type '(or function cons instance character))
+                 (specifier-type '(or function list instance character)))))
+       (or (memq 'fixnum immediates-tested)
+           (not (types-equal-or-intersect (tn-ref-type tn-ref) (specifier-type 'fixnum))))
+       #+64-bit
+       (or (memq single-float-widetag immediates-tested)
+           (not (types-equal-or-intersect (tn-ref-type tn-ref) (specifier-type 'single-float))))))
 
 (defun fixnum-or-other-pointer-tn-ref-p (tn-ref &optional permit-nil)
   (and (sc-is (tn-ref-tn tn-ref) descriptor-reg)
        (not (types-equal-or-intersect
              (tn-ref-type tn-ref)
-             (specifier-type (if permit-nil
-                                 '(or #+64-bit single-float
-                                   function
-                                   cons
-                                   instance
-                                   character)
-                                 '(or #+64-bit single-float
-                                   function
-                                   list
-                                   instance
-                                   character)))))))
+             (if permit-nil
+                 (specifier-type '(or cons . #1=(#+64-bit single-float function cons instance character)))
+                 (specifier-type '(or list . #1#)))))))
+
+;;; Can LOWTAG be distinguished from other tn lowtags by testing a single bit?
+(defun tn-ref-lowtag-bit (lowtag tn-ref &optional permit-nil immediates-tested)
+  (declare (fixnum lowtag))
+  (when tn-ref
+    (let ((type (tn-ref-type tn-ref))
+          (set 0)
+          (clear 0))
+      (when (eq type *universal-type*)
+        (return-from tn-ref-lowtag-bit))
+      (unless (memq 'fixnum immediates-tested)
+        (when (types-equal-or-intersect type (specifier-type 'fixnum))
+          (setf set (logandc2 lowtag-mask fixnum-tag-mask)
+                clear lowtag-mask)))
+      (macrolet ((s (lowtag type)
+                   `(when (and (/= lowtag ,lowtag)
+                               (types-equal-or-intersect type ,(if (symbolp type)
+                                                                   `(specifier-type ',type)
+                                                                   type)))
+                      (setf set (logior set (logand lowtag-mask ,lowtag))
+                            clear (logior clear (logandc2 lowtag-mask ,lowtag))))))
+        #+64-bit
+        (unless (memq single-float-widetag immediates-tested)
+          (s single-float-widetag single-float))
+        (if permit-nil
+            (s list-pointer-lowtag cons)
+            (s list-pointer-lowtag list))
+        (s fun-pointer-lowtag function)
+        (s instance-pointer-lowtag instance)
+        (s character-widetag character)
+        (s other-pointer-lowtag
+           (if permit-nil
+               (specifier-type '(not (or cons . #1=(#+64-bit fixnum single-float function cons instance character))))
+               (specifier-type '(not (or list . #1#)))))
+        (let ((set-bit (logand lowtag-mask (logandc2 lowtag set)))
+              (clear-bit (logandc2 lowtag-mask (logior lowtag clear))))
+          (cond ((plusp set-bit)
+                 (values (sb-kernel::first-bit-set set-bit) 1))
+                ((plusp clear-bit)
+                 (values (sb-kernel::first-bit-set clear-bit) 0))))))))
+
+(defun fun-or-other-pointer-tn-ref-p (tn-ref &optional permit-nil)
+  (and (sc-is (tn-ref-tn tn-ref) descriptor-reg)
+       (not (types-equal-or-intersect
+             (tn-ref-type tn-ref)
+             (if permit-nil
+                 (specifier-type '(or cons instance character fixnum #+64-bit single-float))
+                 (specifier-type '(or cons instance character fixnum #+64-bit single-float)))))))
 
 (defun not-nil-tn-ref-p (tn-ref)
   (not (types-equal-or-intersect (tn-ref-type tn-ref)
