@@ -85,12 +85,13 @@
 
 ;;;; Where to put the different spaces.
 
-(!gencgc-space-setup #x04000000
+(gc-space-setup #x04000000
                      :read-only-space-size 0
                      :dynamic-space-start
                      #+linux   #x4f000000
                      #+netbsd  #x4f000000
-                     #+openbsd #x4f000000)
+                     #+openbsd #x4f000000
+                     #+darwin  #x10000000)
 
 (defconstant alien-linkage-table-growth-direction :up)
 (defconstant alien-linkage-table-entry-size 16)
