@@ -697,6 +697,7 @@
            (length :scs (any-reg immediate))
            (words :scs (any-reg immediate)))
     (:results (result :scs (descriptor-reg) :from :load))
+    (:node-var node)
     (:vop-var vop)
     (:arg-types #+ubsan (:constant t)
                 positive-fixnum positive-fixnum positive-fixnum)
@@ -722,6 +723,15 @@
               (if (sc-is length immediate) (fixnumize (tn-value length)) length))
         (store-originating-pc rax))
       (let ((size (calc-size-in-bytes words result)))
+        (when (sb-c::make-vector-check-overflow-p node)
+          (let ((overflow (generate-error-code vop 'stack-allocated-object-overflows-stack-error size)))
+            (inst sub rsp-tn size)
+            (inst cmp :qword rsp-tn (thread-slot-ea thread-control-stack-start-slot))
+            ;; avoid clearing condition codes
+            (inst lea rsp-tn (if (integerp size)
+                                 (ea size rsp-tn)
+                                 (ea rsp-tn size)))
+            (inst jmp :be overflow)))
         ;; Compute tagged pointer sooner than later since access off RSP
         ;; requires an extra byte in the encoding anyway.
         (stack-allocation size other-pointer-lowtag result
@@ -731,7 +741,6 @@
         ;; in case the length in words is 0, which stores into the LENGTH slot
         ;; as if it were element -1 of data (which probably can't happen).
         (store-string-trailing-null result type length words)
-        ;; FIXME: It would be good to check for stack overflow here.
         (put-header result other-pointer-lowtag type length nil nil)
         )
       #+ubsan
@@ -805,10 +814,19 @@
     (:results (result :scs (descriptor-reg) :from :load))
     (:arg-types positive-fixnum *)
     (:policy :fast-safe)
+    (:node-var node)
+    (:vop-var vop)
     (:temporary (:sc descriptor-reg) tail next limit)
     (:generator 20
       (let ((size (calc-size-in-bytes length next))
             (loop (gen-label)))
+        (when (sb-c::make-list-check-overflow-p node)
+          (let ((overflow (generate-error-code vop 'stack-allocated-object-overflows-stack-error size)))
+            (inst sub rsp-tn size)
+            (inst cmp :qword rsp-tn (thread-slot-ea thread-control-stack-start-slot))
+            ;; avoid clearing condition codes
+            (inst lea rsp-tn (ea rsp-tn size))
+            (inst jmp :be overflow)))
         (stack-allocation size list-pointer-lowtag result)
         (compute-end)
         (inst mov next result)
@@ -916,9 +934,18 @@
                           (inst or temp layout)))
                    temp)
                  result 0 fun-pointer-lowtag (not stack-allocate-p))
-        (inst lea temp (rip-relative-ea label (ash simple-fun-insts-offset word-shift)))
+        (inst lea (pc-size vop)
+              temp (rip-relative-ea label (ash simple-fun-insts-offset word-shift)))
         (storew temp result closure-fun-slot fun-pointer-lowtag)))))
 
+(define-vop (reference-closure)
+  (:info label)
+  (:results (result :scs (descriptor-reg)))
+  (:vop-var vop)
+  (:generator 1
+    (inst lea (pc-size vop)
+      result (rip-relative-ea label fun-pointer-lowtag))))
+
 ;;; The compiler likes to be able to directly make value cells.
 (define-vop (make-value-cell)
   (:args (value :scs (descriptor-reg any-reg immediate constant) :to :result))
@@ -1074,37 +1101,43 @@
 
 #+sb-xc-host
 (define-vop (alloc-code)
-  (:args (total-words :scs (unsigned-reg) :target rdi)
-         (boxed-words :scs (unsigned-reg) :target rsi))
-  (:temporary (:sc unsigned-reg :offset rdi-offset
-               :from (:argument 0) :to :result) rdi)
+  (:args (total-words :scs (unsigned-reg) :target c-arg-1)
+         (boxed-words :scs (unsigned-reg) :target c-arg-2))
+  (:temporary (:sc unsigned-reg
+               :offset #.(first *c-call-register-arg-offsets*)
+               :from (:argument 0) :to :result) c-arg-1)
   (:temporary (:sc unsigned-reg :offset rsi-offset
-               :from (:argument 1) :to :result) rsi)
+               :offset #.(second *c-call-register-arg-offsets*)
+               :from (:argument 1) :to :result) c-arg-2)
   (:temporary (:sc unsigned-reg :offset r15-offset) dummy)
   (:results (res :scs (descriptor-reg)))
   (:ignore dummy)
   (:generator 1
-    (move rdi total-words) ; C arg 1
-    (move rsi boxed-words) ; C arg 2
-    (with-registers-preserved (c :except rdi :frame-reg r15)
+    (move c-arg-1 total-words)
+    (move c-arg-2 boxed-words)
+    (with-registers-preserved (c :except #-win32 rdi #+win32 rcx :frame-reg r15)
       (pseudo-atomic ()
-        #-immobile-code (inst call (ea (make-fixup "alloc_code_object" :foreign 8)))
-        #+immobile-code (inst call (make-fixup "alloc_code_object" :foreign)))
-      (move rdi rax-tn))
-    (move res rdi)))
+        (call-c
+         #-immobile-code (ea (make-fixup "alloc_code_object" :foreign 8))
+         #+immobile-code (make-fixup "alloc_code_object" :foreign)))
+      (move c-arg-1 rax-tn))
+    (move res c-arg-1)))
 
 #+immobile-space
-(macrolet ((c-call (name)
+(macrolet ((c-fun (name)
              `(let ((c-fun (make-fixup ,name :foreign)))
-                (inst call (cond ((sb-c::code-immobile-p node) c-fun)
-                                 (t (progn (inst mov rax c-fun) rax)))))))
+                (cond ((sb-c::code-immobile-p node) c-fun)
+                                 (t (progn (inst mov rax c-fun) rax))))))
 (define-vop (!alloc-immobile-fixedobj)
   (:args (size-class :scs (any-reg) :target c-arg1)
          (nwords :scs (any-reg) :target c-arg2)
          (header :scs (any-reg) :target c-arg3))
-  (:temporary (:sc unsigned-reg :from (:argument 0) :to :eval :offset rdi-offset) c-arg1)
-  (:temporary (:sc unsigned-reg :from (:argument 1) :to :eval :offset rsi-offset) c-arg2)
-  (:temporary (:sc unsigned-reg :from (:argument 2) :to :eval :offset rdx-offset) c-arg3)
+  (:temporary (:sc unsigned-reg :from (:argument 0) :to :eval
+               :offset #.(first *c-call-register-arg-offsets*)) c-arg1)
+  (:temporary (:sc unsigned-reg :from (:argument 1) :to :eval
+               :offset #.(second *c-call-register-arg-offsets*)) c-arg2)
+  (:temporary (:sc unsigned-reg :from (:argument 2) :to :eval
+               :offset #.(third *c-call-register-arg-offsets*)) c-arg3)
   (:temporary (:sc unsigned-reg :from :eval :to (:result 0) :offset rax-offset) rax)
   (:results (result :scs (descriptor-reg)))
   (:node-var node)
@@ -1116,7 +1149,7 @@
    ;; which has that effect
    (inst and rsp-tn -16)
    (pseudo-atomic ()
-     (c-call "alloc_immobile_fixedobj")
+     (call-c (c-fun "alloc_immobile_fixedobj"))
      (move result rax))))
 
 ;;; Timing test:
