@@ -133,8 +133,8 @@
 
 
 (deftransform %alien-funcall ((function type &rest args) * * :node node)
-  (aver (sb-c::constant-lvar-p type))
-  (let* ((type (sb-c::lvar-value type))
+  (aver (sb-c:constant-lvar-p type))
+  (let* ((type (sb-c:lvar-value type))
          (env (sb-c::node-lexenv node))
          (arg-types (alien-fun-type-arg-types type))
          (result-type (alien-fun-type-result-type type)))
@@ -201,9 +201,8 @@
     (foldable flushable movable))
 
 (defoptimizer (sign-extend derive-type) ((x size))
-  (declare (ignore x))
-  (when (sb-c::constant-lvar-p size)
-    (specifier-type `(signed-byte ,(sb-c::lvar-value size)))))
+  (when (sb-c:constant-lvar-p size)
+    (specifier-type `(signed-byte ,(sb-c:lvar-value size)))))
 
 (define-vop (sign-extend)
   (:translate sign-extend)
@@ -224,6 +223,9 @@
     (16 (sign-extend x size))
     (32 (sign-extend x size))))
 
+;;; Note that if jumping _to_ the linkage entry, the jump is to the JMP instruction
+;;; at entry + 0, but if jumping _via_ the linkage index, we can jump to [entry+8]
+;;; which holds the ultimate address to jump to.
 (define-vop (foreign-symbol-sap)
   (:translate foreign-symbol-sap)
   (:policy :fast-safe)
@@ -232,8 +234,16 @@
   (:info foreign-symbol)
   (:results (res :scs (sap-reg)))
   (:result-types system-area-pointer)
+  (:vop-var vop)
   (:generator 2
-   (inst mov res (make-fixup foreign-symbol :foreign))))
+    #-immobile-space ; non-relocatable alien linkage table
+    (inst mov res (make-fixup foreign-symbol :foreign))
+    #+immobile-space ; relocatable alien linkage table
+    (cond ((sb-c::code-immobile-p vop)
+           (inst lea res (rip-relative-ea (make-fixup foreign-symbol :foreign))))
+          (t
+           (inst mov res (thread-slot-ea thread-alien-linkage-table-base-slot))
+           (inst lea res (ea (make-fixup foreign-symbol :alien-code-linkage-index) res))))))
 
 (define-vop (foreign-symbol-dataref-sap)
   (:translate foreign-symbol-dataref-sap)
@@ -243,8 +253,16 @@
   (:info foreign-symbol)
   (:results (res :scs (sap-reg)))
   (:result-types system-area-pointer)
+  (:vop-var vop)
   (:generator 2
-   (inst mov res (ea (make-fixup foreign-symbol :foreign-dataref)))))
+    #-immobile-space ; non-relocatable alien linkage table
+    (inst mov res (ea (make-fixup foreign-symbol :foreign-dataref)))
+    #+immobile-space ; relocatable alien linkage table
+    (cond ((sb-c::code-immobile-p vop)
+           (inst mov res (rip-relative-ea (make-fixup foreign-symbol :foreign-dataref))))
+          (t
+           (inst mov res (thread-slot-ea thread-alien-linkage-table-base-slot))
+           (inst mov res (ea (make-fixup foreign-symbol :alien-data-linkage-index) res))))))
 
 #+sb-safepoint
 (defconstant thread-saved-csp-offset (- (1+ sb-vm::thread-header-slots)))
@@ -264,10 +282,10 @@
       (append
        (loop for gpr in gprs
              collect `(:temporary (:sc any-reg :offset ,gpr :from :eval :to :result)
-                                  ,(car (push (sb-xc:gensym) vars))))
+                                  ,(car (push (gensym) vars))))
        (loop for float to 15
              collect `(:temporary (:sc single-reg :offset ,float :from :eval :to :result)
-                                  ,(car (push (sb-xc:gensym) vars))))
+                                  ,(car (push (gensym) vars))))
        `((:ignore ,@vars))))))
 
 (define-vop (call-out)
@@ -281,13 +299,17 @@
   (:temporary (:sc unsigned-reg :offset rax-offset :to :result) rax)
   #+sb-safepoint
   (:temporary (:sc unsigned-stack :from :eval :to :result) pc-save)
+  #+win32
+  (:temporary (:sc unsigned-reg :offset r15-offset :from :eval :to :result) r15)
   (:ignore results)
   (:vop-var vop)
   (:generator 0
     (move rbx function)
     (emit-c-call vop rax rbx args
                  sb-alien::*alien-fun-type-varargs-default*
-                 #+sb-safepoint pc-save))
+                 #+sb-safepoint pc-save
+                 #+win32 rbx))
+  #+win32 (:ignore r15)
   . #.(destroyed-c-registers))
 
 ;;; Calls to C can generally be made without loading a register
@@ -299,13 +321,26 @@
   (:temporary (:sc unsigned-reg :offset rax-offset :to :result) rax)
   #+sb-safepoint
   (:temporary (:sc unsigned-stack :from :eval :to :result) pc-save)
+  #+win32
+  (:temporary (:sc unsigned-reg :offset r15-offset :from :eval :to :result) r15)
+  #+win32
+  (:ignore r15)
+  #+win32
+  (:temporary (:sc unsigned-reg :offset rbx-offset :from :eval :to :result) rbx)
   (:ignore results)
   (:vop-var vop)
   (:generator 0
-    (emit-c-call vop rax c-symbol args varargsp #+sb-safepoint pc-save))
+    (emit-c-call vop rax c-symbol args varargsp
+                 #+sb-safepoint pc-save
+                 #+win32 rbx))
   . #.(destroyed-c-registers))
 
-(defun emit-c-call (vop rax fun args varargsp #+sb-safepoint pc-save)
+#+win32
+(defconstant win64-seh-direct-thunk-addr win64-seh-data-addr)
+#+win32
+(defconstant win64-seh-indirect-thunk-addr (+ win64-seh-data-addr 8))
+
+(defun emit-c-call (vop rax fun args varargsp #+sb-safepoint pc-save #+win32 rbx)
   (declare (ignorable varargsp))
   ;; Current PC - don't rely on function to keep it in a form that
   ;; GC understands
@@ -332,15 +367,15 @@
   ;; for vararg calls.
   (when varargsp
     (move-immediate rax
-                  (loop for tn-ref = args then (tn-ref-across tn-ref)
-                        while tn-ref
-                        count (eq (sb-name (sc-sb (tn-sc (tn-ref-tn tn-ref))))
-                                  'float-registers))))
+                    (loop for tn-ref = args then (tn-ref-across tn-ref)
+                          while tn-ref
+                          count (eq (sb-name (sc-sb (tn-sc (tn-ref-tn tn-ref))))
+                                    'float-registers))))
 
   ;; Store SP in thread struct, unless the enclosing block says not to
   #+sb-safepoint
   (when (policy (sb-c::vop-node vop) (/= sb-c:insert-safepoints 0))
-    (storew rsp-tn thread-base-tn thread-saved-csp-offset))
+    (inst mov (thread-slot-ea thread-saved-csp-offset) rsp-tn))
 
   #+win32 (inst sub rsp-tn #x20)       ;MS_ABI: shadow zone
 
@@ -348,10 +383,37 @@
   ;; table jump, and from dynamic space we use "CALL [ea]" format
   ;; where ea is the address of the linkage table entry's operand.
   ;; So while the former is a jump to a jump, we can optimize out
-  ;; one jump in a statically linked executable.
-  (inst call (cond ((tn-p fun) fun)
-                   ((sb-c::code-immobile-p vop) (make-fixup fun :foreign))
-                   (t (ea (make-fixup fun :foreign 8)))))
+  ;; one jump in an ELF executable.
+  ;; N.B.: if you change how the call is emitted, you will also have to adjust
+  ;; the UNDEFINED-ALIEN-TRAMP lisp asm routine to recognize the various shapes
+  ;; this instruction sequence can take.
+  #-win32
+  (inst call (if (tn-p fun)
+                 fun
+                 #-immobile-space (ea (make-fixup fun :foreign 8))
+                 #+immobile-space
+                 (cond ((sb-c::code-immobile-p vop) (make-fixup fun :foreign))
+                       (t
+                        ;; Pick r10 as the lowest unused clobberable register.
+                        ;; RAX has a designated purpose, and RBX is nonvolatile (not always
+                        ;; spilled by Lisp because a C function has to save it if used)
+                        (inst mov r10-tn (thread-slot-ea thread-alien-linkage-table-base-slot))
+                        (ea (make-fixup fun :alien-code-linkage-index 8) r10-tn)))))
+
+  ;; On win64, we don't support immobile space (yet) and calls go through one of
+  ;; the thunks defined in set_up_win64_seh_data(). If the linkage table is
+  ;; involved, RBX either points to a linkage table trampoline or to the linkage
+  ;; table operand; this simplifies UNDEFINED-ALIEN-TRAMP's job.
+  #+win32
+  (cond ((tn-p fun)
+         (move rbx fun)
+         (inst mov rax win64-seh-direct-thunk-addr)
+         (inst call rax))
+        (t
+         (inst mov rbx (make-fixup fun :foreign 8))
+         (inst mov rax win64-seh-indirect-thunk-addr)
+         (inst call rax)))
+
   ;; For the undefined alien error
   (note-this-location vop :internal-error)
   #+win32 (inst add rsp-tn #x20)       ;MS_ABI: remove shadow space
@@ -359,8 +421,7 @@
   ;; Zero the saved CSP, unless this code shouldn't ever stop for GC
   #+sb-safepoint
   (when (policy (sb-c::vop-node vop) (/= sb-c:insert-safepoints 0))
-    (inst xor (object-slot-ea thread-base-tn thread-saved-csp-offset 0)
-          rsp-tn)))
+    (inst xor (thread-slot-ea thread-saved-csp-offset) rsp-tn)))
 
 (define-vop (alloc-number-stack-space)
   (:info amount)
@@ -389,17 +450,6 @@
           (inst sub :qword (alien-stack-ptr) delta)))
       (inst mov result (alien-stack-ptr)))))
 
-;;; not strictly part of the c-call convention, but needed for the
-;;; WITH-PINNED-OBJECTS macro used for "locking down" lisp objects so
-;;; that GC won't move them while foreign functions go to work.
-(define-vop (touch-object)
-  (:translate touch-object)
-  (:args (object))
-  (:ignore object)
-  (:policy :fast-safe)
-  (:arg-types t)
-  (:generator 0))
-
 ;;; Callbacks
 
 #-sb-xc-host
@@ -456,7 +506,7 @@
                      ;; stack location to a temporary register.
                      (unless gpr
                        (incf stack-argument-count)
-                       (setf gpr temp-reg-tn)
+                       (setf gpr rax)
                        (inst mov gpr stack-arg-tn))
                      ;; Copy from either argument register or temporary
                      ;; register to target.
@@ -473,8 +523,8 @@
                             ;; temporary (general purpose) register, and
                             ;; from there to the target location.
                             (incf stack-argument-count)
-                            (inst mov temp-reg-tn stack-arg-tn)
-                            (inst mov target-tn temp-reg-tn)))))
+                            (inst mov rax stack-arg-tn)
+                            (inst mov target-tn rax)))))
                   (t
                    (bug "Unknown alien floating point type: ~S" type)))))
 
@@ -500,8 +550,7 @@
           (inst call rax)
 
           ;; Back! Restore frame
-          (inst mov rsp rbp)
-          (inst pop rbp))
+          (inst leave))
 
         #+sb-thread
         (progn
@@ -521,11 +570,13 @@
           #+win32 (inst sub rsp #x20)
           #+win32 (inst and rsp #x-20)
           ;; Call
-          (inst mov rax (foreign-symbol-address "callback_wrapper_trampoline"))
-          (inst call rax)
+          #+immobile-space (inst call (static-symbol-value-ea 'callback-wrapper-trampoline))
+          ;; do this without MAKE-FIXUP because fixup'ing does not happen when
+          ;; assembling callbacks (probably could, but ...)
+          #-immobile-space
+          (inst call (ea (+ (foreign-symbol-address "callback_wrapper_trampoline") 8)))
           ;; Back! Restore frame
-          (inst mov rsp rbp)
-          (inst pop rbp))
+          (inst leave))
 
         ;; Result now on top of stack, put it in the right register
         (cond
