@@ -41,7 +41,7 @@
 #include <signal.h>
 /* #include <sys/sysinfo.h> */
 #include "validate.h"
-#include "gc-internal.h"
+#include "gc.h"
 
 
 
@@ -255,8 +255,6 @@ os_alloc_gc_space(int space_id, int attributes, os_vm_address_t addr, os_vm_size
  * any OS-dependent special low-level handling for signals
  */
 
-#if defined LISP_FEATURE_GENCGC
-
 /*
  * The GENCGC needs to be hooked into whatever signal is raised for
  * page fault on this OS.
@@ -271,13 +269,13 @@ memory_fault_handler(int signal, siginfo_t *siginfo, os_context_t *context)
     os_restore_tls_segment_register(context);
 #endif
 
-    FSHOW((stderr, "Memory fault at: %p, PC: %p\n", fault_addr, OS_CONTEXT_PC(context)));
-
 #ifdef LISP_FEATURE_SB_SAFEPOINT
     if (handle_safepoint_violation(context, fault_addr)) return;
 #endif
 
+#ifdef LISP_FEATURE_GENCGC
     if (gencgc_handle_wp_violation(context, fault_addr)) return;
+#endif
 
     if (!handle_guard_page_triggered(context,fault_addr))
             lisp_memory_fault_error(context, fault_addr);
@@ -301,28 +299,6 @@ os_install_interrupt_handlers(void)
     }
 }
 
-#else /* Currently PPC/Darwin/Cheney only */
-
-static void
-sigsegv_handler(int signal, siginfo_t *info, os_context_t *context)
-{
-    os_vm_address_t addr;
-
-    addr = arch_get_bad_addr(signal, info, context);
-    if (cheneygc_handle_wp_violation(context, addr)) return;
-
-    if (!handle_guard_page_triggered(context, addr))
-            interrupt_handle_now(signal, info, context);
-}
-
-void
-os_install_interrupt_handlers(void)
-{
-    ll_install_handler(SIG_MEMORY_FAULT, sigsegv_handler);
-}
-
-#endif /* defined GENCGC */
-
 #ifdef __NetBSD__
 static void netbsd_init()
 {
