@@ -10,6 +10,7 @@
  */
 
 #include "sbcl.h"
+#include "lispobj.h"
 
 #include <stdio.h>
 #include <sys/types.h>
@@ -34,10 +35,13 @@
 #include "lispregs.h"
 #include "interrupt.h"
 #include "thread.h"
+#include "genesis/constants.h"
+#include "genesis/instance.h"
 #include "genesis/static-symbols.h"
 #include "genesis/primitive-objects.h"
 #include "genesis/gc-tables.h"
-#include "gc-internal.h"
+#include "genesis/unwind-block.h"
+#include "gc.h"
 #include "../../tlsf-bsd/tlsf/tlsf.h"
 extern void* tlsf_control;
 
@@ -89,7 +93,7 @@ struct crash_thread_preamble {
 };
 
 // Prevent some mixups in case you add fields to the crash dump
-const int CRASH_PREAMBLE_SIGNATURE =
+const uword_t CRASH_PREAMBLE_SIGNATURE =
     (sizeof (struct crash_preamble) << 16) | sizeof (struct crash_thread_preamble);
 
 #if defined LISP_FEATURE_X86 || defined LISP_FEATURE_X86_64 // un-tested elsewhere
@@ -176,7 +180,7 @@ void save_gc_crashdump(char *pathname,
         if (ici) {
 #ifdef LISP_FEATURE_C_STACK_IS_CONTROL_STACK
             sp = *os_context_register_addr(threadcontext, reg_SP);
-#else
+#elif defined reg_CSP
             sp = *os_context_register_addr(threadcontext, reg_CSP);
 #endif
         } else if (th->state_word.state == STATE_DEAD) {
@@ -233,7 +237,7 @@ static cmd call_cmd, dump_cmd, print_cmd, quit_cmd, help_cmd;
 static cmd flush_cmd, regs_cmd, exit_cmd;
 static cmd print_context_cmd, pte_cmd, search_cmd;
 static cmd backtrace_cmd, catchers_cmd;
-static cmd threads_cmd;
+static cmd threads_cmd, findpath_cmd, layouts_cmd;
 
 extern void gc_stop_the_world(), gc_start_the_world();
 static void suspend_other_threads() {
@@ -252,7 +256,7 @@ static int save_cmd(char **ptr) {
     char *name  = parse_token(ptr);
     if (!name) {
         fprintf(stderr, "Need filename\n");
-        return 1;
+        return 0;
     }
 #if (defined LISP_FEATURE_X86 || defined LISP_FEATURE_X86_64) && defined LISP_FEATURE_SB_THREAD
     suspend_other_threads();
@@ -270,11 +274,11 @@ void list_lisp_threads(int regions) {
     for_each_thread(th) {
         memcpy(&pthread, &th->os_thread, N_WORD_BYTES);
         struct thread_instance* i = (void*)(th->lisp_thread - INSTANCE_POINTER_LOWTAG);
-        char* name = NULL;
-        if (i->name != NIL &&
-            widetag_of(native_pointer(i->name)) == SIMPLE_BASE_STRING_WIDETAG)
-            name = (char*)VECTOR(i->name)->data;
-        fprintf(stderr, "%p %p %p \"%s\"\n", th, pthread, (void*)i, name);
+        lispobj name = instancep(th->lisp_thread) ? i->_name : 0;
+        char* cname = NULL;
+        // it's OK to call widetag_of on NIL but not NULL
+        if (name && simple_base_string_p(name)) cname = vector_sap(name);
+        fprintf(stderr, "%p %p %p \"%s\"\n", th, pthread, (void*)i, cname);
         if (regions) {
 #define show_tlab(label, r) fprintf(stderr, "  %s @ %p: %p %p %p\n", label, \
          &th->r, th->r.start_addr, th->r.end_addr, th->r.free_pointer)
@@ -285,11 +289,74 @@ void list_lisp_threads(int regions) {
 #undef show_tlab
         }
     }
+    if (regions) {
+#define show_tlab(label, r) fprintf(stderr, "  %s @ %p: %p %p %p\n", label, \
+         r, r->start_addr, r->end_addr, r->free_pointer)
+        fprintf(stderr, "global regions:\n");
+        show_tlab("mixed    ", mixed_region);
+        show_tlab("smallmix ", small_mixed_region);
+        show_tlab("unboxed  ", unboxed_region);
+        show_tlab("code     ", code_region);
+        show_tlab("boxed    ", boxed_region);
+        show_tlab("cons     ", cons_region);
+#undef show_tlab
+    }
 }
 static int threads_cmd(char **ptr) {
     list_lisp_threads(more_p(ptr) && !strncmp(*ptr, "-r", 2));
     return 0;
 }
+extern int heap_trace_verbose;
+extern int gc_pathfind_aux(lispobj*, lispobj, lispobj, lispobj, int);
+
+static int findpath_cmd(char **ptr) {
+    // prevent the path finder from seeing the object that results from parsing the command
+    lispobj* stackptr = (lispobj*)&ptr;
+    // overaligned for 32-bit but doesn't matter
+    struct vector __attribute__ ((aligned (16))) result;
+    struct cons __attribute__ ((aligned (16))) list;
+    /* Despite capturing the stack pointer coming in to this function,
+     * it's very difficult to ensure that a locally allocated weak pointer
+     * is not above that (and hence its value slot seen).
+     * So it has to be malloc()ed. */
+    char* wp_mem = malloc(sizeof (struct weak_pointer) + N_WORD_BYTES);
+    struct weak_pointer* wp =
+      (void*)(wp_mem + (((uword_t)wp_mem & LOWTAG_MASK) ? N_WORD_BYTES : 0));
+    wp->header = ((WEAK_POINTER_SIZE-1)<<N_WIDETAG_BITS)|WEAK_POINTER_WIDETAG;
+    if (parse_lispobj(ptr, &wp->value)) {
+        list.car = make_lispobj(wp, OTHER_POINTER_LOWTAG);
+        list.cdr = NIL;
+        result.header = SIMPLE_VECTOR_WIDETAG;
+        result.length_ = make_fixnum(1);
+        result.data[0] = 0;
+        suspend_other_threads();
+        int save = heap_trace_verbose;
+        heap_trace_verbose = 4;
+        gc_pathfind_aux(stackptr,
+                        make_lispobj(&list, LIST_POINTER_LOWTAG),
+                        make_lispobj(&result, OTHER_POINTER_LOWTAG),
+                        0, 2);
+        heap_trace_verbose = save;
+        unsuspend_other_threads();
+        lispobj path = result.data[0];
+        if (listp(path)) {
+            fprintf(stderr, "Answer:\n");
+            while (path != NIL) {
+                struct cons* pair = CONS(CONS(path)->car);
+                if (listp(pair->cdr)) {
+                    // thread root - complicated to print
+                } else {
+                    // otherwise, object and word index
+                    fprintf(stderr, " %"OBJ_FMTX" word %d\n",
+                            pair->car, (int)pair->cdr);
+                }
+                path = CONS(path)->cdr;
+            }
+        }
+    }
+    free(wp_mem);
+    return 0;
+}
 static int verify_cmd(char __attribute__((unused)) **ptr) {
     gencgc_verbose = 1;
     suspend_other_threads();
@@ -300,7 +367,6 @@ static int verify_cmd(char __attribute__((unused)) **ptr) {
 static int gc_cmd(char **ptr) {
     int last_gen = 0;
     extern generation_index_t verify_gens;
-    extern boolean pre_verify_gen_0;
     if (more_p(ptr)) parse_number(ptr, &last_gen);
     gencgc_verbose = 2;
     pre_verify_gen_0 = 1;
@@ -311,21 +377,22 @@ static int gc_cmd(char **ptr) {
     return 0;
 }
 
-static int tlsf_cmd(__attribute__((unused)) char **ptr) {
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
+static int tlsf_cmd(__attribute__((unused)) char **ptr) {
     tlsf_dump_pool(tlsf_control, tlsf_mem_start, "/dev/tty");
 #ifdef TLSF_CONFIG_DEBUG
     tlsf_check(tlsf_control);
     tlsf_check_pool(tlsf_mem_start);
-#endif
 #endif
     return 0;
 }
+#endif
 
 static struct cmd {
     char *cmd, *help;
     int (*fn)(char **ptr);
 } supported_cmds[] = {
+    // Commands with no help string are all at-your-own-risk
     {"help", "Display this help information.", help_cmd},
     {"?", "(an alias for help)", help_cmd},
     {"backtrace", "Backtrace up to N frames.", backtrace_cmd},
@@ -335,18 +402,22 @@ static struct cmd {
     {"dump", "Dump memory starting at ADDRESS for COUNT words.", dump_cmd},
     {"d", "(an alias for dump)", dump_cmd},
     {"exit", "Exit this instance of the monitor.", exit_cmd},
+    {"findpath", "Find path to an object.", findpath_cmd},
     {"flush", "Flush all temp variables.", flush_cmd},
+    {"layouts", "Dump LAYOUT instances.", layouts_cmd},
     {"print", "Print object at ADDRESS.", print_cmd},
     {"p", "(an alias for print)", print_cmd},
     {"pte", "Page table entry for address", pte_cmd},
     {"quit", "Quit.", quit_cmd},
     {"regs", "Display current Lisp registers.", regs_cmd},
     {"search", "Search heap for object.", search_cmd},
-    {"save", "Produce crashdump", save_cmd},
+    {"save", 0, save_cmd}, // snapshot heap to file ("best effort" though)
     {"threads", "List threads", threads_cmd},
-    {"tlsfdump", "Dump TLSF structures", tlsf_cmd},
+#ifdef LISP_FEATURE_IMMOBILE_SPACE
+    {"tlsfdump", 0, tlsf_cmd}, // (unsafely) dump TLSF structures
+#endif
     {"verify", "Check heap invariants", verify_cmd},
-    {"gc", "Collect garbage", gc_cmd},
+    {"gc", 0, gc_cmd}, // (this is not the right way to invoke GC)
     {NULL, NULL, NULL}
 };
 
@@ -359,7 +430,7 @@ visible(unsigned char c)
         return c;
 }
 
-static boolean valid_widetag_p(unsigned char widetag) {
+static bool valid_widetag_p(unsigned char widetag) {
     // TODO: ensure that widetag is defined (not "unused") and is for a headered object
     // (i.e. is not CHARACTER_WIDETAG and not some other things)
     return other_immediate_lowtag_p(widetag);
@@ -403,7 +474,7 @@ dump_cmd(char **ptr)
         count = -count;
     }
 
-    boolean aligned = ((uword_t)addr & LOWTAG_MASK) == 0;
+    bool aligned = ((uword_t)addr & LOWTAG_MASK) == 0;
     if (decode && (!aligned || displacement < 0)) {
         printf("Sorry, can only decode if aligned and stepping forward\n");
         decode = 0;
@@ -434,13 +505,22 @@ dump_cmd(char **ptr)
                    visible(cptr[0]), visible(cptr[1]),
                    visible(cptr[2]), visible(cptr[3]));
 #endif
-#ifdef LISP_FEATURE_GENCGC
+#ifdef LISP_FEATURE_GENERATIONAL
             if (aligned) {
                 lispobj ptr = *(lispobj*)addr;
                 int gen;
                 if (is_lisp_pointer(ptr) && gc_managed_heap_space_p(ptr)
-                    && (gen = gc_gen_of(ptr, 99)) != 99) // say that static is 99
+                    && (gen = gc_gen_of(ptr, 99)) != 99) { // say that static is 99
                     if (gen != 99) printf(" | %d", gen);
+                } else {
+                    printf("    "); // padding to make MR part line up
+                }
+            }
+#endif
+#ifdef LISP_FEATURE_MARK_REGION_GC
+            if (aligned && find_page_index(addr) != -1) {
+                extern bool allocation_bit_marked(void*);
+                printf(" %c", allocation_bit_marked(addr) ? '*' : ' ');
             }
 #endif
             if (decode && addr == (char*)next_object) {
@@ -503,17 +583,13 @@ regs_cmd(char __attribute__((unused)) **ptr)
     printf("\n");
 #endif
 
-#ifdef LISP_FEATURE_GENCGC
+#ifdef LISP_FEATURE_GENERATIONAL
     /* printf("DYNAMIC\t=\t%p\n", (void*)DYNAMIC_SPACE_START); */
 #else
     printf("STATIC\t=\t%p   ", static_space_free_pointer);
     printf("RDONLY\t=\t%p   ", read_only_space_free_pointer);
     printf("DYNAMIC\t=\t%p\n", (void*)current_dynamic_space);
 #endif
-
-#ifndef LISP_FEATURE_GENCGC
-    printf("TRIGGER\t=\t%p\n", (void*)current_auto_gc_trigger);
-#endif
     return 0;
 }
 
@@ -640,6 +716,10 @@ print_context(os_context_t *context)
         brief_print((lispobj)(*os_context_register_addr(context,i)));
 
     }
+#if defined(LISP_FEATURE_DARWIN) && defined(LISP_FEATURE_PPC)
+    printf("DAR:\t\t 0x%08lx\n", (unsigned long)(*os_context_register_addr(context, 41)));
+    printf("DSISR:\t\t 0x%08lx\n", (unsigned long)(*os_context_register_addr(context, 42)));
+#endif
 #ifndef REG_PC
     printf("PC:\t\t  0x%08lx\n", (unsigned long)os_context_pc(context));
 #endif
@@ -733,7 +813,72 @@ catchers_cmd(char __attribute__((unused)) **ptr)
     return 0;
 }
 
-extern boolean gc_active_p;
+struct layout_collection {
+    struct cons* list;
+    int passno;
+};
+static int count_layout_occurs(lispobj x, struct cons* list)
+{
+    int ct = 0;
+    for ( ; list ; list = (void*)list->cdr ) if (list->car == x) ++ct;
+    return ct;
+}
+
+static uword_t display_layouts(lispobj* where, lispobj* limit, uword_t arg)
+{
+    extern struct vector * classoid_name(lispobj * classoid);
+    struct layout_collection *lc = (void*)arg;
+    where = next_object(where, 0, limit); /* find first marked object */
+    for ( ; where ; where = next_object(where, object_size(where), limit) ) {
+        if (widetag_of(where) == INSTANCE_WIDETAG &&
+            instance_layout(where) != 0 &&
+            layout_depth2_id(LAYOUT(instance_layout(where))) == LAYOUT_LAYOUT_ID) {
+            struct layout* l = (void*)where;
+            struct classoid* c = (void*)native_pointer(l->classoid);
+            if (lc->passno == 1) {
+                // on the first pass, just collect the classoids;
+                struct cons* cons = malloc(sizeof (struct cons));
+                cons->car = (lispobj)c;
+                cons->cdr = (lispobj)lc->list;
+                lc->list = cons;
+            } else {
+                // print some information preceded by a '*' if more than one
+                // layout points to the classoid of this layout
+                int count = count_layout_occurs((lispobj)c, lc->list);
+                struct vector* v = classoid_name((lispobj*)c);
+                char* name =
+                  header_widetag(v->header)==SIMPLE_BASE_STRING_WIDETAG ? (char*)v->data : "?";
+                fprintf(stderr, "%c %p %16" OBJ_FMTX " %16" OBJ_FMTX " %p %" OBJ_FMTX " %s\n",
+                        count>1 ? '*' : ' ', l, l->clos_hash, l->uw_id_word0,
+                        c, l->invalid, name);
+
+            }
+        }
+    }
+   return 0;
+}
+
+static int layouts_cmd(char __attribute__((unused)) **ptr)
+{
+    fprintf(stderr, "Dup, Layout, Hash, ID_Word, Classoid, Invalid, Name\n");
+    struct layout_collection lc;
+    lc.list = 0;
+    for (lc.passno = 1; lc.passno <= 2; ++lc.passno) {
+        walk_generation(display_layouts, -1, (uword_t)&lc);
+#ifdef LISP_FEATURE_IMMOBILE_SPACE
+        display_layouts((lispobj*)FIXEDOBJ_SPACE_START, fixedobj_free_pointer,
+                        (uword_t)&lc);
+#endif
+    }
+    struct cons* l = lc.list;
+    while (l) {
+        struct cons* next = (struct cons*)l->cdr;
+        free(l);
+        l = next;
+    }
+    return 0;
+}
+
 extern FILE *gc_activitylog_file;
 void
 ldb_monitor(void)
@@ -797,40 +942,33 @@ ldb_monitor(void)
     }
 }
 
-/* what we do when things go badly wrong at a low level */
-void
-monitor_or_something()
-{
-    ldb_monitor();
-}
-
 #ifdef STANDALONE_LDB
 void gc_stop_the_world() { } // do nothing
 void gc_start_the_world() { } // do nothing
 #include <errno.h>
 #include "core.h"
-#include "gencgc-private.h"
 struct lisp_startup_options lisp_startup_options;
+char gc_coalesce_string_literals;
 
 void unwind_binding_stack() { lose("Can't unwind binding stack"); }
 FILE *prepare_to_save(__attribute__((unused)) char *filename,
-                      __attribute__((unused)) boolean prepend_runtime,
+                      __attribute__((unused)) bool prepend_runtime,
                       __attribute__((unused)) void **runtime_bytes,
                       __attribute__((unused)) size_t *runtime_size) {
     lose("Can't prepare_to_save");
 }
-boolean save_runtime_to_filehandle(__attribute__((unused)) FILE *output,
-                                   __attribute__((unused)) void *runtime,
-                                   __attribute__((unused)) size_t runtime_size,
-                                   __attribute__((unused)) int application_type) {
+bool save_runtime_to_filehandle(__attribute__((unused)) FILE *output,
+                                __attribute__((unused)) void *runtime,
+                                __attribute__((unused)) size_t runtime_size,
+                                __attribute__((unused)) int application_type) {
     lose("Can't save_runtime_to_filehandle");
 }
-boolean save_to_filehandle(__attribute__((unused)) FILE *file,
-                           __attribute__((unused)) char *filename,
-                           __attribute__((unused)) lispobj init_function,
-                           __attribute__((unused)) boolean make_executable,
-                           __attribute__((unused)) boolean save_runtime_options,
-                           __attribute__((unused)) int core_compression_level) {
+bool save_to_filehandle(__attribute__((unused)) FILE *file,
+                        __attribute__((unused)) char *filename,
+                        __attribute__((unused)) lispobj init_function,
+                        __attribute__((unused)) bool make_executable,
+                        __attribute__((unused)) bool save_runtime_options,
+                        __attribute__((unused)) int core_compression_level) {
     lose("Can't save_to_filehandle");
 }
 
@@ -980,7 +1118,7 @@ int load_gc_crashdump(char* pathname)
         *os_context_register_addr(context,reg_SP) = (os_context_register_t)stackptr;
 #endif
         gc_assert(*os_context_register_addr(context,reg_SP) == (os_context_register_t)stackptr);
-#else
+#elif defined reg_CSP
         *os_context_register_addr(context, reg_CSP) = (uword_t)stackptr;
 #endif
         checked_read(" stack", fd, stackptr, thread_preamble.control_stack_nbytes);
@@ -993,10 +1131,9 @@ int load_gc_crashdump(char* pathname)
         checked_read(" TLS", fd, &th->lisp_thread, preamble.tls_size-skip);
         write_TLS(FREE_INTERRUPT_CONTEXT_INDEX, make_fixnum(1), th);
         struct thread_instance* instance = (void*)(th->lisp_thread - INSTANCE_POINTER_LOWTAG);
-        lispobj name = instance->name;
-        char* cname = (gc_managed_addr_p(name) &&
-                       widetag_of(native_pointer(name)) == SIMPLE_BASE_STRING_WIDETAG)
-                      ? (char*)name+1 : 0;
+        lispobj name = instance->_name;
+        char* cname = gc_managed_addr_p(name) && simple_base_string_p(name)
+                      ? vector_sap(name) : 0;
         fprintf(stderr, "thread @ %p originally %p, %d bind_stk words, %d val_stk words '%s'\n",
                 th, (void*)thread_preamble.address,
                 (int)(thread_preamble.binding_stack_nbytes>>WORD_SHIFT),
@@ -1049,7 +1186,7 @@ int main(int argc, char *argv[], char **envp)
         fprintf(stderr, "Usage: ldb crashdump\n");
         return 1;
     }
-    boolean have_hardwired_spaces = os_preinit(argv, envp);
+    bool have_hardwired_spaces = os_preinit(argv, envp);
     allocate_lisp_dynamic_space(have_hardwired_spaces);
     gc_init();
     load_gc_crashdump(argv[1]);
