@@ -24,7 +24,7 @@
 #include "pseudo-atomic.h"
 #include "genesis/instance.h"
 #include "genesis/gc-tables.h"
-#include "genesis/list-node.h"
+#include "genesis/split-ordered-list.h"
 #include "thread.h"
 
 /* forward declarations */
@@ -44,18 +44,6 @@ int n_gcs;
  * and only a few rare messages are printed at level 1. */
 int gencgc_verbose = 0;
 
-/* FIXME: At some point enable the various error-checking things below
- * and see what they say. */
-
-/* We hunt for pointers to old-space, when GCing generations >= verify_gen.
- * Set verify_gens to HIGHEST_NORMAL_GENERATION + 2 to disable this kind of
- * check. */
-generation_index_t verify_gens = HIGHEST_NORMAL_GENERATION + 2;
-
-/* Should we do a pre-scan of the heap before it's GCed? */
-int pre_verify_gen_0 = 0; // FIXME: should be named 'pre_verify_gc'
-
-
 /*
  * GC structures and variables
  */
@@ -953,7 +941,8 @@ garbage_collect_generation(generation_index_t generation, int raise,
     // been on pages that got freed by free_oldspace.
     dynspace_codeblob_tree_snapshot = 0;
     if (generation >= verify_gens)
-        verify_heap(cur_thread_approx_stackptr, VERIFY_POST_GC | (generation<<16));
+        hexdump_and_verify_heap(cur_thread_approx_stackptr,
+                                VERIFY_POST_GC | (generation<<1) | raise);
 
     extern int n_unboxed_instances;
     n_unboxed_instances = 0;
@@ -1207,7 +1196,6 @@ collect_garbage(generation_index_t last_gen)
                      && (generation_average_age(gen)
                          > generations[gen].minimum_age_before_gc))));
 
-#ifdef LISP_FEATURE_SOFT_CARD_MARKS
     {
     // Turn sticky cards marks to the regular mark.
     page_index_t page;
@@ -1219,7 +1207,6 @@ collect_garbage(generation_index_t last_gen)
             gcm[card] = (gcm[card] == STICKY_MARK) ? CARD_MARKED : gcm[card];
     }
     }
-#endif
 
     /* Save the high-water mark before updating next_free_page */
     if (next_free_page > high_water_mark)
@@ -1554,33 +1541,6 @@ static bool card_markedp(void* addr)
     return gc_card_mark[addr_to_card_index(addr)] != CARD_UNMARKED;
 }
 
-char *page_card_mark_string(page_index_t page, char *result)
-{
-    long card = addr_to_card_index(page_address(page));
-    if (cardseq_all_marked_nonsticky(card))
-        result[0] = '*', result[1] = 0;
-    else if (!cardseq_any_marked(card))
-        result[0] = '-', result[1] = 0;
-    else {
-        int i;
-        for(i=0; i<CARDS_PER_PAGE; ++i)
-        switch (gc_card_mark[card+i] & MARK_BYTE_MASK) {
-        case CARD_MARKED: result[i] = '*'; break;
-        case CARD_UNMARKED: result[i] = '-'; break;
-#ifdef LISP_FEATURE_SOFT_CARD_MARKS
-        case STICKY_MARK: result[i] = 'S'; break;
-#else
-        case WP_CLEARED_AND_MARKED: result[i] = 'd'; break; // "d" is for dirty
-#endif
-        default: result[i] = '?'; break; // illegal value
-        }
-        result[CARDS_PER_PAGE] = 0;
-    }
-    return result;
-}
-
-#define PRINT_HEADER_ON_FAILURE 2048
-
 // Check a single pointer. Return 1 if we should stop verifying due to too many errors.
 // (Otherwise continue showing errors until then)
 // NOTE: This function can produces false failure indications,
@@ -1590,10 +1550,10 @@ static void note_failure(lispobj thing, lispobj *where, struct verify_state *sta
                          char *str)
 {
     lock();
-    if (state->flags & PRINT_HEADER_ON_FAILURE) {
+    if (state->flags & VERIFY_PRINT_HEADER_ON_FAILURE) {
         if (state->flags & VERIFY_PRE_GC) fprintf(stderr, "pre-GC failure\n");
         if (state->flags & VERIFY_POST_GC) fprintf(stderr, "post-GC failure\n");
-        state->flags &= ~PRINT_HEADER_ON_FAILURE;
+        state->flags &= ~VERIFY_PRINT_HEADER_ON_FAILURE;
     }
     if (state->object_addr) {
         lispobj obj = compute_lispobj(state->object_addr);
@@ -1616,8 +1576,6 @@ static void note_failure(lispobj thing, lispobj *where, struct verify_state *sta
 static int
 verify_pointer(lispobj thing, lispobj *where, struct verify_state *state)
 {
-    extern int valid_lisp_pointer_p(lispobj);
-
     /* Strict containment: no pointer from a heap space may point
      * to anything outside of a heap space. */
     // bool strict_containment = state->flags & VERIFY_FINAL;
@@ -1707,23 +1665,14 @@ verify_pointer(lispobj thing, lispobj *where, struct verify_state *state)
          *    (either/or because Lisp marks the header card,
          *     but the collector marks the cell's card.) */
         int marked = card_markedp(where)
-#ifdef LISP_FEATURE_SOFT_CARD_MARKS
         || (state->object_header
             && header_widetag(state->object_header) != SIMPLE_VECTOR_WIDETAG
-            && card_markedp(state->object_addr))
-#elif defined LISP_FEATURE_EXECUTABLE_FUNINSTANCES
-        /* #+(and (not soft-card-marks) executable-funinstances) could find the mark
-         * for a page-spanning funinstance on the preceding page, because it uses
-         * logical marking, not physical protection of the page holding the pointer */
-        || (header_widetag(state->object_header) == FUNCALLABLE_INSTANCE_WIDETAG
-            && card_markedp(state->object_addr))
-#endif
-        ;
+            && card_markedp(state->object_addr));
         FAIL_IF(!marked, "younger obj from WP page");
     }
     int valid;
     if (state->flags & VERIFY_AGGRESSIVE) // Extreme paranoia mode
-        valid = valid_lisp_pointer_p(thing);
+        valid = valid_tagged_pointer_p(thing);
     else {
         /* Efficiently decide whether 'thing' is plausible.
          * This MUST NOT use properly_tagged_descriptor_p() which
@@ -1780,6 +1729,7 @@ static int verify_headered_object(lispobj* object, sword_t nwords,
         }
 #endif
         for (i=2; i <nheader_words; ++i) CHECK(object[i], object+i);
+#ifndef NDEBUG // avoid "unused" warnings on auto vars of for_each_simple_fun()
         // Check the SIMPLE-FUN headers
         for_each_simple_fun(i, fheaderp, code, 1, {
 #if defined LISP_FEATURE_COMPACT_INSTANCE_HEADER
@@ -1789,6 +1739,7 @@ static int verify_headered_object(lispobj* object, sword_t nwords,
             gc_assert((fheaderp->header >> 32) == 0);
 #endif
         });
+#endif
 #if 0 // this looks redundant. It's checked with each pointer, no?
         bool rememberedp = header_rememberedp(code->header);
         /* The remembered set invariant is that an object is marked "written"
@@ -2004,11 +1955,12 @@ int verify_heap(__attribute__((unused)) lispobj* cur_thread_approx_stackptr,
     if (verbose)
         fprintf(stderr,
                 flags & VERIFY_PRE_GC ? "Verify before GC" :
-                flags & VERIFY_POST_GC ? "Verify after GC(%d)" :
+                flags & VERIFY_POST_GC ? "Verify after GC(%d,%d)" :
                 "Heap check", // if called at a random time
-                (int)(flags>>16)); // generation number
+                (flags >> 1) & 7, // generation number
+                flags & 1); // 'raise'
     else
-        state.flags |= PRINT_HEADER_ON_FAILURE;
+        state.flags |= VERIFY_PRINT_HEADER_ON_FAILURE;
 
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
 #  ifdef __linux__
@@ -2107,11 +2059,6 @@ void gc_show_pte(lispobj obj)
     printf("not in GC'ed space\n");
 }
 
-int hexdump_and_verify_heap(lispobj* cur_thread_approx_stackptr, int flags)
-{
-    return verify_heap(cur_thread_approx_stackptr, flags);
-}
-
 /* The other implementation of gc_gen_report_to_file gets the generations
  * of small objects wrong, as it only checks the generations of pages. */
 void gc_gen_report_to_file(int filedes, FILE *file) {
