@@ -722,3 +722,34 @@
                                                        x))
                                              (length x)))))
                  `(values (integer 11 12) &optional))))
+
+(with-test (:name :aref-dimension-checking)
+  (checked-compile-and-assert
+      (:optimize :safe)
+      `(lambda (x)
+         (aref x 0))
+    ((#2A((1 2) (3 4))) (condition 'type-error))))
+
+(with-test (:name :aref-constant-type-derive)
+  (flet ((test (form type)
+           (assert
+            (type-specifiers-equal
+             (caddr
+              (sb-kernel:%simple-fun-type
+               (checked-compile
+                `(lambda (a)
+                   ,form))))
+             `(values ,type &optional)))))
+    (test `(aref #(1 2 3) a)
+          '(integer 1 3))
+    (test `(svref #(1 2 3.0) a)
+          '(or (integer 1 2) single-float))
+    (test `(aref ,(make-array 3 :fill-pointer 2 :initial-contents #(1 2 3.0)) a)
+          '(or (integer 1 2) single-float))))
+
+(with-test (:name :make-array-initial-contents-zero-dimensions)
+  (checked-compile-and-assert
+      (:optimize :safe)
+      `(lambda (d)
+         (make-array d :initial-contents 1))
+    ((nil) #0a1 :test #'equalp)))
