@@ -577,6 +577,28 @@
                  :initial-value 0))
     (("abc") 294)))
 
+(with-test (:name :reduce-initial-value-from-end)
+  (checked-compile-and-assert
+   ()
+   `(lambda (s)
+      (reduce #'funcall s :from-end t :initial-value '(1)))
+   (('(car)) 1))
+  (checked-compile-and-assert
+   ()
+   `(lambda (s e)
+      (reduce #'funcall s :from-end e :initial-value '(1)))
+   (('(car) t) 1))
+  (checked-compile-and-assert
+   ()
+   `(lambda (e)
+      (reduce #'funcall #*1 :from-end e :initial-value 1 :key (lambda (x) x 'list)))
+   ((t) '(1) :test #'equal))
+  (checked-compile-and-assert
+   ()
+   `(lambda ()
+      (reduce #'funcall #*1 :from-end t :initial-value 1 :key (lambda (x) x 'list)))
+   (() '(1) :test #'equal)))
+
 (with-test (:name :get-defined-fun-lambda-list-error)
   (assert (nth-value 1 (checked-compile '(lambda () (defun x 10)) :allow-failure t))))
 
@@ -693,3 +715,29 @@
                          (vector-push-extend 1 x))
                       :allow-warnings 'warning))))
 
+(with-test (:name :defmethod-malformed-let*)
+  (assert (nth-value 5
+                     (checked-compile
+                      `(lambda ()
+                         (cl:defmethod ,(gensym) ()
+                           (let* ((a 1 2))
+                             a)))
+                      :allow-failure t))))
+
+(with-test (:name :position-derive-empty-type)
+  (multiple-value-bind (fun failure warning)
+      (checked-compile
+       `(lambda (s)
+          (position #\a (the simple-string s) :start 4 :end 2))
+       :allow-warnings t)
+    (declare (ignore failure))
+    (assert warning)
+    (assert-error (funcall fun "abcdef") sb-kernel:bounding-indices-bad-error)))
+
+(with-test (:name :cast-movement-empty-types)
+  (assert (nth-value 2
+                     (checked-compile
+                      `(lambda ()
+                         (loop for x to 2
+                               sum (the cons (signum x))))
+                      :allow-warnings 'warning))))
