@@ -32,7 +32,7 @@
                           elsewhere-pc expect-elsewhere-pc))))))))
     (assert ok)))
 
-;;; Check that valid_lisp_pointer_p is correct for all pure boxed objects
+;;; Check that valid_tagged_pointer_p is correct for all pure boxed objects
 ;;; using the super quick check of header validity.
 (defun randomly-probe-pure-boxed-objects ()
   (let (list)
@@ -41,9 +41,8 @@
        (declare (ignore widetag))
        (let* ((index
                (sb-vm::find-page-index (sb-kernel:get-lisp-obj-address obj)))
-              (type (ldb (byte 6 (+ #+big-endian 2))
-                         (sb-alien:slot (sb-alien:deref sb-vm::page-table index)
-                                        'sb-vm::flags))))
+              (type (sb-alien:slot (sb-alien:deref sb-vm::page-table index)
+                                   'sb-vm::flags)))
          ;; mask off the SINGLE_OBJECT and OPEN_REGION bits
          (when (and (eq (logand type 7) 2) ; PAGE_TYPE_BOXED
                     ;; Cons cells on boxed pags are page filler
@@ -60,10 +59,10 @@
                  (base (logandc2 taggedptr sb-vm:lowtag-mask)))
             (dotimes (i 40)
               (let* ((ptr (+ base (random size)))
-                     (valid (sb-di::valid-lisp-pointer-p (sb-sys:int-sap ptr))))
+                     (valid (sb-di::valid-tagged-pointer-p (sb-sys:int-sap ptr))))
                 (if (= ptr taggedptr)
                     (assert (= valid 1))
                     (assert (= valid 0)))))))))))
 (compile 'randomly-probe-pure-boxed-objects)
-(with-test (:name :fast-valid-lisp-pointer-p)
+(with-test (:name :fast-valid-tagged-pointer-p)
   (randomly-probe-pure-boxed-objects))
