@@ -99,7 +99,8 @@
     (dotimes (i (1- (length a)))
       (assert (= (aref a (1+ i)) (1+ (aref a i)))))))
 
-(with-test (:name :list-allocated-objects)
+(with-test (:name :list-allocated-objects
+            :skipped-on :weak-vector-readbarrier) ; uses more weak-pointers
   ;; Assert that if :COUNT is supplied as a higher number
   ;; than number of objects that exists, the output is
   ;; not COUNT many items long.
@@ -161,7 +162,7 @@
 (with-test (:name :static-fdefn-space)
   (sb-int:dovector (name sb-vm:+static-fdefns+)
     (assert (eq (sb-ext:heap-allocated-p (sb-int:find-fdefn name))
-                (or #+immobile-code :immobile :static)))))
+                (or #+(and immobile-code x86-64) :immobile :static)))))
 
 ;;; SB-EXT:GENERATION-* accessors returned bogus values for generation > 0
 (with-test (:name :bug-529014)
