@@ -117,7 +117,7 @@
     (when show (show (list-head *lfl*) #'get-next "del "))))
 
 ;; Enable heap validity tester
-#+gencgc (setf (sb-alien:extern-alien "verify_gens" char) 0)
+#+generational (setf (sb-alien:extern-alien "verify_gens" char) 0)
 
 (test-util:with-test (:name :lockfree-list-gc-correctness)
   ;; Create a small list and perform logical deletion of 2 nodes
@@ -253,6 +253,7 @@
   (gc)
   (assert (= (sb-kernel:generation-of *5*) 0))
   (assert (= (sb-kernel:generation-of *10*) 0))
+  #-mark-region-gc
   (assert (not (eql (get-lisp-obj-address *10*)
                     *addr-of-10*))))
 
