@@ -973,3 +973,14 @@
 
 (with-test (:name :dx-pathname-parts-dont-crash)
   (prin1 (pathname-peristence-test)))
+
+(with-test (:name :internal-pathname-hash-incorporates-version)
+  (opaque-identity
+   (loop for i below 1000
+         collect (make-pathname :host "SYS" :name "FOO" :type "LISP" :version i))))
+
+(with-test (:name :pathname-hash-more-strongly)
+  ;; examples from xof's email
+  (assert (/= (sxhash #P"APPVA") (sxhash #P"APPKZ")))
+  (assert (/= (sxhash #P"AVERA") (sxhash #P"AVEQR")))
+  (assert (/= (sxhash #P"AVERB") (sxhash #P"AVEQS"))))
