@@ -1,8 +1,3 @@
-#+(and linux sb-thread 64-bit)
-(sb-alien:alien-funcall (sb-alien:extern-alien
-                         "reset_gc_stats"
-                         (function sb-alien:void)))
-
 (load "test-util.lisp")
 (load "assertoid.lisp")
 (load "compiler-test-util.lisp")
@@ -54,6 +49,9 @@
              (t
               (push (merge-pathnames (parse-namestring arg)) *explicit-test-files*))))
   (setf *explicit-test-files* (nreverse *explicit-test-files*))
+  ;; FIXME: randomizing the order tests are run in, especially the "pure" ones,
+  ;; might help detect accidental side-effects by inducing failures elsewhere.
+  ;; And/or try all permutations using an infinite number of machines.
   (with-open-file (log "test.log" :direction :output
                        :if-exists :supersede
                        :if-does-not-exist :create)
@@ -200,6 +198,7 @@
     `(sb-c::*code-serialno*
       sb-c::*compile-elapsed-time*
       sb-c::*compile-file-elapsed-time*
+      sb-impl::**finalizer-store**
       sb-impl::*finalizer-rehashlist*
       sb-impl::*finalizers-triggered*
       sb-impl::*all-packages*
@@ -214,8 +213,10 @@
       ,(maybe "SB-KERNEL" "*EVAL-CALLS*")
       sb-kernel::*type-cache-nonce*
       sb-ext:*gc-run-time*
+      sb-ext:*gc-real-time*
       sb-kernel::*gc-epoch*
       sb-int:*n-bytes-freed-or-purified*
+      ,(maybe "SB-APROF" "*ALLOCATION-PROFILE-METADATA*")
       ,(maybe "SB-VM" "*BINDING-STACK-POINTER*")
       ,(maybe "SB-VM" "*CONTROL-STACK-POINTER*")
       ,(maybe "SB-THREAD" "*JOINABLE-THREADS*")
